// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Plots the current hour open, high, and low values.
    /// </summary>
    [Description("Plots the current hour open, high, and low values.")]
    [Gui.Design.DisplayName("Current Hour OHL")]
    public class CurrentHourOHL : Indicator
    {
        #region Variables
		private double		currentOpen			=	0;
        private double		currentHigh			=	0;
		private double		currentLow			=	0;
		private bool		plotCurrentValue	=	false;
		private bool		showOpen			=	true;
		private bool		showHigh			=	true;
		private bool		showLow				=	true;
		// When time interval wraps back to zero, draw a line.
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Orange, 2), PlotStyle.Square, "Current Open"));
			Add(new Plot(new Pen(Color.Green, 2), PlotStyle.Square, "Current High"));
			Add(new Plot(new Pen(Color.Red, 2), PlotStyle.Square, "Current Low"));
			Plots[0].Pen.DashStyle = DashStyle.Dash;
			Plots[1].Pen.DashStyle = DashStyle.Dash;
			Plots[2].Pen.DashStyle = DashStyle.Dash;
			
			AutoScale 			= false;
            CalculateOnBarClose	= false;
            Overlay				= true;
			
			}
		
		public int wrapMinute(int minute) {
			if( minute > 59) { return minute - 60; }
			if( minute < 0) { return minute + 60; }
			return minute;
		}
		
		public bool HourBreak() {
			if( CurrentBar > 5) {
				int prevMin = Time[2].Minute;
				int currMin = Time[1].Minute;//
//				if( Time[1].DayOfWeek == DayOfWeek.Sunday) {
//					prevMin = wrapMinute( prevMin - 30);
//					currMin = wrapMinute( currMin - 30);
//				}
				return currMin < prevMin || Time[1].DayOfWeek < Time[2].DayOfWeek;
			} else {
				return false;
			}
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (Bars.CurrentBar < 10) { return; }
			
			if ((Bars.Period.Id == PeriodType.Minute && Bars.Period.Value <= 30) ||
				(Bars.Period.Id == PeriodType.Second && Bars.Period.Value <= 1800))
			{
				// If the Minute has gone from 55 to 05 that means we just 
				// wrapped to a new hour!
				if(HourBreak())
				{
					currentOpen 	= 	Open[0];
					currentHigh 	= 	High[0];
					currentLow		=	Low[0];
				}
	
				currentHigh 	= 	Math.Max(currentHigh, High[0]);
				currentLow		= 	Math.Min(currentLow, Low[0]);
	
				if (ShowOpen)
				{
					if (!PlotCurrentValue || !HourBreak())
						CurrentOpen.Set(currentOpen);
					else
						for (int idx = 0; idx < CurrentOpen.Count; idx++)
							CurrentOpen.Set(idx, currentOpen);
				}
	
				if (ShowHigh)
				{
					if (!PlotCurrentValue || currentHigh != High[0])
						CurrentHigh.Set(currentHigh);
					else
						for (int idx = 0; idx < CurrentHigh.Count; idx++)
							CurrentHigh.Set(idx, currentHigh);
				}
	
				if (ShowLow)
				{
					if (!PlotCurrentValue || currentLow != Low[0])
						CurrentLow.Set(currentLow);
					else
						for (int idx = 0; idx < CurrentLow.Count; idx++)
							CurrentLow.Set(idx, currentLow);
				}
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CurrentOpen
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CurrentHigh
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries CurrentLow
        {
            get { return Values[2]; }
        }

		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Show open")]
        public bool ShowOpen
        {
            get { return showOpen; }
			set { showOpen = value; }
        }
		
		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Show high")]
        public bool ShowHigh
        {
            get { return showHigh; }
			set { showHigh = value; }
        }

		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Plot current value only")]
		public bool PlotCurrentValue
		{
			get { return plotCurrentValue; }
			set { plotCurrentValue = value; }
		}
		
		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Show low")]
        public bool ShowLow
        {
            get { return showLow; }
			set { showLow = value; }
        }
        #endregion
    }
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CurrentHourOHL[] cacheCurrentHourOHL = null;

        private static CurrentHourOHL checkCurrentHourOHL = new CurrentHourOHL();

        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        public CurrentHourOHL CurrentHourOHL()
        {
            return CurrentHourOHL(Input);
        }

        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        public CurrentHourOHL CurrentHourOHL(Data.IDataSeries input)
        {

            if (cacheCurrentHourOHL != null)
                for (int idx = 0; idx < cacheCurrentHourOHL.Length; idx++)
                    if (cacheCurrentHourOHL[idx].EqualsInput(input))
                        return cacheCurrentHourOHL[idx];

            CurrentHourOHL indicator = new CurrentHourOHL();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            CurrentHourOHL[] tmp = new CurrentHourOHL[cacheCurrentHourOHL == null ? 1 : cacheCurrentHourOHL.Length + 1];
            if (cacheCurrentHourOHL != null)
                cacheCurrentHourOHL.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCurrentHourOHL = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CurrentHourOHL CurrentHourOHL()
        {
            return _indicator.CurrentHourOHL(Input);
        }

        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        public Indicator.CurrentHourOHL CurrentHourOHL(Data.IDataSeries input)
        {
            return _indicator.CurrentHourOHL(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CurrentHourOHL CurrentHourOHL()
        {
            return _indicator.CurrentHourOHL(Input);
        }

        /// <summary>
        /// Plots the current hour open, high, and low values.
        /// </summary>
        /// <returns></returns>
        public Indicator.CurrentHourOHL CurrentHourOHL(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CurrentHourOHL(input);
        }

    }
}
#endregion
