/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO Buy Zone
/// Version: v1.01
/// Converted to NinjaTrader by cvax
/// 
/// Changelog: v1.01 - Improved painting method of Buy Zones
/// 
/// Updated to be reset hourly by Wayne Walter
/// DONATIONS AND GIFTS ACCEPTED
/// PayPal email = admin@waynewalter.com
/// 
/// If you send a donation, I'll reply with another one
/// with more features.
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{

    [Description("Plots the buy zones for an instrument and resets every hour.")]
    [Gui.Design.DisplayName("TRO Hourly Buy Zone")]
    public class TROHourlyBuyZone : Indicator
    {
        #region Variables
        // Wizard generated variables
		private double		spread				= .03;	//Spread between top and bottom of zones
		private double		buyzone				= .01;	//Offset from current hour open
		private bool		fill				= true;		//Fill color inbetween zones
		private bool		drawopen			= true;
		
        // User defined variables (add any user defined variables below)
		private double		longtop				= 0;
		private double		longbot				= 0;
		private double		shorttop			= 0;
		private double		shortbot			= 0;
		private int 		lastHourBreak		= -1;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Blue, 1), PlotStyle.Hash, "Long Zone: Bottom"));
			Add(new Plot(new Pen(Color.Black, 1), PlotStyle.Hash, "Open"));
			Add(new Plot(new Pen(Color.Red , 1), PlotStyle.Hash, "Short Zone: Top"));
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= true;
			AutoScale			= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1)	return;
			
			longtop = CurrentHourOHL().CurrentOpen[0]+spread+buyzone;
			longbot = CurrentHourOHL().CurrentOpen[0]+spread;
			shorttop = CurrentHourOHL().CurrentOpen[0]-spread;
			shortbot = CurrentHourOHL().CurrentOpen[0]-spread-buyzone;
			
			LongBottom.Set(longbot);
			if (drawopen)
				OpenValue.Set(CurrentHourOHL().CurrentOpen[0]);
			ShortTop.Set(shorttop);

			if( CurrentHourOHL().HourBreak()) {
				lastHourBreak = CurrentBar;
			}
			
			// This paints between the lines.
			if ( CurrentBar > lastHourBreak) {
				int barsAgo = CurrentBar-lastHourBreak;
				if( lastHourBreak >= 0 && barsAgo > 0) {
					DrawRegion("tagtop"+lastHourBreak, barsAgo, 0, LongBottom, longtop, Color.Empty, Color.Blue, 2);
					DrawRegion("tagbot"+lastHourBreak, barsAgo, 0, ShortTop, shortbot, Color.Empty, Color.Red, 2);
				}
			}
		

        }
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LongBottom
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OpenValue
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ShortTop
        {
            get { return Values[2]; }
        }
		
		[Description("Width between the top and bottom of the Buy Zones")]
		[Category("Parameters")]
		public double BuyZoneWidth
		{
			get { return spread; }
			set { spread = Math.Max(0, value); }
		}
		
		[Description("Buy Zone's offset from the current hour Open")]
		[Category("Parameters")]
		public double BuyZoneOffset
		{
			get { return buyzone; }
			set { buyzone = Math.Max(0, value); }
		}
		
		[Description("Fill color the Buy Zones.")]
		[Category("Plots")]
		public bool ColorZone
		{
			get { return fill; }
			set { fill = value; }
		}
		
		[Description("Draw the Open line.")]
		[Category("Plots")]
		public bool DrawOpen
		{
			get { return drawopen; }
			set { drawopen = value; }
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TROHourlyBuyZone[] cacheTROHourlyBuyZone = null;

        private static TROHourlyBuyZone checkTROHourlyBuyZone = new TROHourlyBuyZone();

        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        public TROHourlyBuyZone TROHourlyBuyZone(double buyZoneOffset, double buyZoneWidth)
        {
            return TROHourlyBuyZone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        public TROHourlyBuyZone TROHourlyBuyZone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            checkTROHourlyBuyZone.BuyZoneOffset = buyZoneOffset;
            buyZoneOffset = checkTROHourlyBuyZone.BuyZoneOffset;
            checkTROHourlyBuyZone.BuyZoneWidth = buyZoneWidth;
            buyZoneWidth = checkTROHourlyBuyZone.BuyZoneWidth;

            if (cacheTROHourlyBuyZone != null)
                for (int idx = 0; idx < cacheTROHourlyBuyZone.Length; idx++)
                    if (Math.Abs(cacheTROHourlyBuyZone[idx].BuyZoneOffset - buyZoneOffset) <= double.Epsilon && Math.Abs(cacheTROHourlyBuyZone[idx].BuyZoneWidth - buyZoneWidth) <= double.Epsilon && cacheTROHourlyBuyZone[idx].EqualsInput(input))
                        return cacheTROHourlyBuyZone[idx];

            TROHourlyBuyZone indicator = new TROHourlyBuyZone();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.BuyZoneOffset = buyZoneOffset;
            indicator.BuyZoneWidth = buyZoneWidth;
            indicator.SetUp();

            TROHourlyBuyZone[] tmp = new TROHourlyBuyZone[cacheTROHourlyBuyZone == null ? 1 : cacheTROHourlyBuyZone.Length + 1];
            if (cacheTROHourlyBuyZone != null)
                cacheTROHourlyBuyZone.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTROHourlyBuyZone = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TROHourlyBuyZone TROHourlyBuyZone(double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TROHourlyBuyZone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        public Indicator.TROHourlyBuyZone TROHourlyBuyZone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TROHourlyBuyZone(input, buyZoneOffset, buyZoneWidth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TROHourlyBuyZone TROHourlyBuyZone(double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TROHourlyBuyZone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument and resets every hour.
        /// </summary>
        /// <returns></returns>
        public Indicator.TROHourlyBuyZone TROHourlyBuyZone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TROHourlyBuyZone(input, buyZoneOffset, buyZoneWidth);
        }

    }
}
#endregion
